/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.protocol.server.CollectionAdapter;
import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.TargetType;
import org.apache.abdera.protocol.server.impl.AbstractWorkspaceProvider;
import org.apache.abdera.protocol.server.impl.SimpleWorkspaceInfo;
import org.apache.abdera.protocol.server.impl.TemplateTargetBuilder;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.jdbc.InMemoryEmbeddedRegistry;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

public class RegistryProvider extends AbstractWorkspaceProvider {
    RegistryAdapter adapter;
    public static final String baseURI = "/wso2registry";

    public CollectionAdapter getCollectionAdapter(RequestContext request) {
        return adapter;
    }

    public RegistryProvider() throws Exception {
        RegistryContext context = RegistryContext.getRegistryContext();
        EmbeddedRegistry embeddedRegistry = null;
        if (context != null) {
            embeddedRegistry = context.getEmbeddedRegistry();
        } else {
            InputStream is = null;
            try {
                is = new FileInputStream("registry.xml");
            } catch (FileNotFoundException e) {
                // no problemo
            }
            RegistryContext ctx = new RegistryContext(is);
            RegistryContext.setRegistryContext(ctx);
        }

        if (embeddedRegistry == null) {
            RegistryRealm realm = new InMemoryRegistryRealm();
            embeddedRegistry = new InMemoryEmbeddedRegistry(realm);
        }

        RegistryResolver resolver = new RegistryResolver(embeddedRegistry, baseURI);
        adapter = new RegistryAdapter(resolver, embeddedRegistry);
        
        this.setTargetResolver(resolver);

        this.setTargetBuilder(
                new TemplateTargetBuilder().setTemplate(TargetType.TYPE_COLLECTION,
                                                        "{target_base}/atom/{collection}").
                        setTemplate(RegistryResolver.TAGS_TYPE, "{target_base}/atom/{collection};tags"));
        SimpleWorkspaceInfo workspace = new SimpleWorkspaceInfo();
        workspace.setTitle("A Simple Workspace");
        workspace.addCollection(adapter);
        addWorkspace(workspace);
    }
}
