/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.utils.creators.DatabaseCreator;
import org.wso2.registry.jdbc.utils.creators.DatabaseCreatorFactory;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;

import javax.sql.DataSource;

public class EmbeddedRegistry {

    private JDBCRegistry jdbcRegistry;
    private UserRealm defaultRealm;
    protected RegistryContext registryContext;

    public EmbeddedRegistry(RegistryContext context) throws RegistryException {
        this.registryContext = context;
        configure();
    }

    public EmbeddedRegistry(RegistryContext context, UserRealm realm) throws RegistryException {
        this.registryContext = context;
        configure(realm);
    }

    protected EmbeddedRegistry() {
    }

    protected void configure() throws RegistryException {
        DataSource dataSource = registryContext.getDataSource();

        DatabaseCreator databaseCreator = DatabaseCreatorFactory.getDatabaseCreator(dataSource);
        databaseCreator.createRegistryDatabase();
        databaseCreator.createUserManagerDatabase();

        defaultRealm = new RegistryRealm(dataSource);
        jdbcRegistry = new JDBCRegistry(registryContext, defaultRealm);
    }

    protected void configure(UserRealm defaultRealm) throws RegistryException {
        this.defaultRealm = defaultRealm;
        configure();
    }

    public UserRegistry getUserRegistry() throws RegistryException {
        return new UserRegistry(
                RegistryConstants.ANONYMOUS_USER,
                RegistryConstants.ANONYMOUS_PASSWORD,
                jdbcRegistry,
                defaultRealm,
                registryContext.getDataSource());
    }

    /**
     * Returns a registry to be used for system operations. Human users should not be allowed
     * log in using this registry.
     *
     * @return User registry for system user.
     * @throws RegistryException
     */
    public UserRegistry getSystemRegistry() throws RegistryException {
        return new UserRegistry(
                RegistryConstants.SYSTEM_USER,
                RegistryConstants.SYSTEM_PASSWORD,
                jdbcRegistry,
                defaultRealm,
                registryContext.getDataSource());
    }

    public UserRegistry getUserRegistry(String userName, String password) throws RegistryException {
        return new UserRegistry(userName, password, jdbcRegistry, defaultRealm,
                                registryContext.getDataSource());
    }

    public UserRegistry getUserRegistry(String userName) throws RegistryException {
        return new UserRegistry(userName, jdbcRegistry, defaultRealm,
                                registryContext.getDataSource());
    }
}
